<?php

namespace Drupal\social_auth_ok\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\social_auth_ok\OkAuthManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\social_api\Plugin\NetworkManager;
use Drupal\social_auth\SocialAuthUserManager;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\social_auth\SocialAuthDataHandler;

/**
 * Class OkAuthController.
 */
class OkAuthController extends ControllerBase {

  /**
   * Drupal\social_api\Plugin\NetworkManager definition.
   *
   * @var \Drupal\social_api\Plugin\NetworkManager
   */
  protected $networkManager;

  /**
   * Drupal\social_auth\SocialAuthUserManager definition.
   *
   * @var \Drupal\social_auth\SocialAuthUserManager
   */
  protected $userManager;


  /**
   * @var \Drupal\social_auth_ok\OkAuthManager
   */
  protected $okManager;

  /**
   * Symfony\Component\HttpFoundation\RequestStack definition.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $request;

  /**
   * Drupal\social_auth\SocialAuthDataHandler definition.
   *
   * @var \Drupal\social_auth\SocialAuthDataHandler
   */
  protected $dataHandler;


  /**
   * Constructs a new OkAuthController object.
   * @param \Drupal\social_api\Plugin\NetworkManager $plugin_network_manager
   * @param \Drupal\social_auth\SocialAuthUserManager $social_auth_user_manager
   * @param \Drupal\social_auth_ok\OkAuthManager $ok_manager
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   * @param \Drupal\social_auth\SocialAuthDataHandler $social_auth_data_handler
   */
  public function __construct(NetworkManager $plugin_network_manager, SocialAuthUserManager $social_auth_user_manager, OkAuthManager $ok_manager, RequestStack $request_stack, SocialAuthDataHandler $social_auth_data_handler) {
    $this->networkManager = $plugin_network_manager;
    $this->userManager = $social_auth_user_manager;
    $this->okManager = $ok_manager;
    $this->request = $request_stack;
    $this->dataHandler = $social_auth_data_handler;
  }


  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.network.manager'),
      $container->get('social_auth.user_manager'),
      $container->get('social_auth_ok.manager'),
      $container->get('request_stack'),
      $container->get('social_auth.data_handler')
    );
  }


  /**
   * Response for path user/login/ok
   * Redirects to OK for authentication
   */
  public function redirectToOk() {
    try {
      $ok = $this->networkManager->createInstance('social_auth_ok')->getSdk();
      if (!$ok) {
        throw new \Exception($this->t('No configuration found for Social Auth Odnoklassniki'));
      }
      $this->okManager->setClient($ok);
      $login_url = $this->okManager->getAuthorizationUrl();
      $state = $this->okManager->getState();
      $this->dataHandler->set('oauth2state', $state);
      $response = new TrustedRedirectResponse($login_url);
    } catch (\Exception $e) {
      \Drupal::logger('Social Auth Odnoklassniki')->error($e->getMessage());
      \Drupal::messenger()->addMessage($this->t('Authentication error, please contact site administrator.'));
      $response = $this->redirect('user.login');
    }

    return $response;
  }


  /**
   *
   * Response for path 'user/login/ok/callback'
   * Callback to return from OK authorization page
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   */
  public function callback() {
    $error = $this->request->getCurrentRequest()->get('error');
    if ($error == 'access_denied') {
      \Drupal::messenger()->addMessage($this->t('You could not be authenticated.'), 'error');
      return $this->redirect('user.login');
    }

    $ok = $this->networkManager->createInstance('social_auth_ok')->getSdk();
    if (!$ok) {
      \Drupal::logger('Social Auth Odnoklassniki')->error($this->t('No configuration found for Social Auth Odnoklassniki'));
      \Drupal::messenger()->addMessage($this->t('Authentication error, please contact site administrator.'));
      return $this->redirect('user.login');
    }

    $state = $this->dataHandler->get('oauth2state');
    $retrievedState = $this->request->getCurrentRequest()->query->get('state');
    if (empty($retrievedState) || ($retrievedState !== $state)) {
      $this->userManager->nullifySessionKeys();
      \Drupal::messenger()->addMessage($this->t('Odnoklassniki login failed. Invalid OAuth2 State.'), 'error');
      return $this->redirect('user.login');
    }

    $this->dataHandler->set('access_token', $this->okManager->getAccessToken());
    $this->okManager->setClient($ok)->authenticate();

    // Gets user's profile from OK API.
    if (!$profile = $this->okManager->getUserInfo()) {
      \Drupal::messenger()->addMessage($this->t('Odnoklassniki login failed, could not load profile. Contact site administrator.'), 'error');
      return $this->redirect('user.login');
    }

    // Gets user's email from the profile.
    $data = $profile->toArray();
    $email = isset($data['email']) ? $data['email'] : '';
    if (!$email) {
      \Drupal::messenger()->addMessage($this->t('Odnoklassniki login failed. This site requires permission to get your email address.'), 'error');
      return $this->redirect('user.login');
    }

    return $this->userManager->authenticateUser($profile->getName(), $email, $profile->getId(), $this->okManager->getAccessToken(), $profile->getImageUrl());
  }

}
